package org.jivesoftware.plugins.instantFeeds;

import org.jivesoftware.util.ParamUtils;
import org.jivesoftware.openfire.XMPPServer;
import org.xmpp.packet.JID;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.ArrayList;

/**
 * Servlet that handles the following REST methods:
 * <ul>
 *  <li>add: add a subscription for a given user to a given feed.</li>
 *  <li>verify: verify a subscription for a given user.</li>
 *  <li>get: get all the subscriptions for a given user</li>
 *  <li>delete: delete a given subscription for a given user</li>
 * </ul>
 *
 */
public class InstantFeedsSubscriptionsHTTPServlet extends HttpServlet {

    private InstantFeedsPlugin plugin;

    public void init() throws ServletException {
        plugin = (InstantFeedsPlugin) XMPPServer.getInstance().getPluginManager().getPlugin("instantfeeds");

    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doPost(request, response);
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        String method = request.getRequestURI().replaceAll("/plugins/instantfeeds/api/http/subscriptions/", "");

        if ("add".equalsIgnoreCase(method)) {
            addSubscription(request, response);

        }
        else if ("verify".equalsIgnoreCase(method)) {
            verifySubscription(request, response);
        }

        else if ("get".equalsIgnoreCase(method)) {
            getSubscriptions(request, response);

        }

        else if ("update".equalsIgnoreCase(method)) {
            // TODO: implement update subscription
        }

        else if ("delete".equalsIgnoreCase(method)) {
            deleteSubscription(request, response);

        }
        
    }

    /**
     * Sign up to receive instant messages for a given feed.  URL parameters:
     * <ul>
     *  <li>url (required) the URL of the feed to get updates from</li>
     *  <li>username (required) the screename (AIM/Yahoo/MSN) or username (XMPP)</li>
     *  <li>servicename (required for non XMPP) the service name (aim, yahoo, msn)</li>
     *  <li>redirect (required) a URL to redirect the user to after subscription confirmation</li>
     *  <li>password (optional) the instantfeeds password the user has established (bypasses
     * confirmation of feed subscription if provided)</li>
     * </ul>
     * A successful response or redirect means only that the feed was added to the queue to be
     * verified. The user will receive an instant message confirming or denying the subscription
     * request after the feed has been verified as valid.
     *
     * @param request the http servlet request
     * @param response the http servlet response
     * @throws IOException if errors occur while writing to response
     */

    private void addSubscription(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String url = ParamUtils.getParameter(request, "url", true);
        String username = ParamUtils.getParameter(request, "username", true);
        String servicename = ParamUtils.getParameter(request, "servicename", true);
        String redirect = ParamUtils.getParameter(request, "redirect", true);
        String password = ParamUtils.getParameter(request, "password", true);

        String jid = null;

        List<String> errors = new ArrayList<String>();

        if (url == null || url.length() == 0) {
            errors.add("error.url.invalid");
        }

        if (username == null || username.length() == 0) {
            errors.add("error.username.invalid");
        }

        if (username != null && username.indexOf("@") == -1) {
            if (servicename == null || servicename.length() == 0) {
                errors.add("error.servicename.invalid");
            }

        }

        if (redirect == null || redirect.length() == 0) {
            errors.add("error.redirect.invalid");
        }

        if (username != null && username.indexOf("@") > -1) {
            jid = username;
        } else if (username != null) {
            jid = username + "@" + servicename + "." + XMPPServer.getInstance().getServerInfo().getName();
        } else {
            errors.add("error.username.invalid");
        }

        // make sure you have a valid JID
        if (jid != null) {
            try {
                JID testJID = new JID(jid);
            } catch (IllegalArgumentException e) {
                errors.add("error.jid.invalid");

            }
        }

        if (errors.size() == 0) {

            // TODO: add password support
            /**
            if (password != null && password.length() > 0 && password.equals(plugin.loadUser(username + "@" + servicename).getPassword()) {
                FeedSubscriptionRequest fsr = new FeedSubscriptionRequest(jid, url);
                plugin.addFeed(fsr);
            }
            **/
            FeedSubscriptionRequest fsr = new FeedSubscriptionRequest(jid, url, redirect);
            plugin.addFeed(fsr);

            writeSuccess(request, response);
        } else {
            writeErrors(request, response, errors);
        }
    }

    /**
     * Verify a requested subscription. URL parameters:
     * <ul>
     *  <li>v (required) the MD5 hash of the feed url and the user's JID.</li>
     * </ul>
     * A successful response or redirect means that the hash was valid and that the user
     * was successfully subscribed to the feed. The user will receive an instant message
     * confirming the subscription.
     *
     * @param request the http servlet request
     * @param response the http servlet response
     * @throws IOException if errors occur while writing to response
     */
    private void verifySubscription(HttpServletRequest request, HttpServletResponse response) throws IOException {
        FeedSubscriptionRequest req = null;
        String code = ParamUtils.getParameter(request, "v", true);

        if (code != null) {
           req = plugin.verifyRequest(code);
        }

        if (req != null) {
            if (req.getConfirmationURL() != null) {
                
                response.sendRedirect(req.getConfirmationURL());
            } else {
                writeSuccess(request, response);
            }
            
        } else {
            List<String> errors = new ArrayList<String>();
            errors.add("error.verificationcode.invalid");
            writeErrors(request, response, errors);
        }
    }


    /**
     * Get a list of subscriptions for a given user. URL parameters:
     * <ul>
     *  <li>username (required)</li>
     *  <li>servicename (required)</li>
     *  <li>password (required)</li>
     * </ul>
     *
     * @param request the http servlet request
     * @param response the http servlet response
     */

    private void getSubscriptions(HttpServletRequest request, HttpServletResponse response) {
        // TODO: implement get subscriptions
    }

    /**
     * Delete a subscription for a given user. URL parameters:
     * <ul>
     *  <ul>url (required)</li>
     *  <li>username (required)</li>
     *  <li>servicename (required)</li>
     *  <li>password (required)</li>
     * </ul>
     * @param request the http servlet request
     * @param response the http servlet request
     */

    private void deleteSubscription(HttpServletRequest request, HttpServletResponse response) {
        // TODO: implement delete subscriptions
    }


    private void writeSuccess(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("text/xml");
        Writer writer = response.getWriter();
        writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        writer.write("<result code=\"done\"/>");
        writer.close();

    }

    private void writeErrors(HttpServletRequest request, HttpServletResponse response, List<String> errors) throws IOException {
        response.setContentType("text/xml");
        Writer writer = response.getWriter();
        writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        writer.write("<result code=\"failure\">");
        for (String error : errors) {
            writer.write("<error code=\"" + error + "\" />");
        }
        writer.write("</result>");
        writer.close();

    }



}
